/* Copyright (C) 1999 Jeff Heflin - University of Maryland

   This software is provided under the terms of the GNU General Public
   License Version 2, June 1991. See the file License.html for details.
*/

package Expose;

import java.net.*;
import java.util.*;

/** A data structure that stores information about a web page, including
  the last time it was visited and cost information used in ranking
  the search. */
public class VisitNode {

  public URL url;
  public Date lastVisited;
  public int nodeCost;       // cost of path including this node
  public int parentCost;     // cost of path to node's parent

  public VisitNode(URL url, int parentCost, int nodeCost) {

    this.url = url;
    this.parentCost = parentCost;
    this.nodeCost = nodeCost;
    this.lastVisited = null;
  }

  public VisitNode(URL url, int parentCost, int nodeCost, Date lastVisited) {

    this.url = url;
    this.parentCost = parentCost;
    this.nodeCost = nodeCost;
    this.lastVisited = lastVisited;
  }

  /* This method is over-ridden so that a SearchList can compare two 
     VisitNode and say they are the same if they refer to the same URL */
  public boolean equals(Object obj) {

    VisitNode anode;

    if (obj instanceof VisitNode) {
      anode = (VisitNode)obj;
      if (this.url.equals(anode.url))
	return true;
    }
    return false;
  }

  public void setNodeCost(int cost) {

    this.nodeCost = cost;
  }

  public void setLastVisited(Date lastVisited) {

    this.lastVisited = lastVisited;
  }
}
