/* Copyright (C) 1999 PLUS Group - University of Maryland
 *	
 * This software is provided under the terms of the GNU Lesser General Public
 * License Version 2.1, February 1999. See the header in ParkaNet.java
 * and the file License.html for details.
 */

package  ParkaClient;

import   java.util.*;

/** Extends Response in order to deal with queries for BrowserWin
   information
 */
public class BrowserResponse  extends Response
{
    private  String        browserResponse;
    private  String[][]    allTokens;
    private  boolean       lastColFinished;
    private  int           numCols;
    private  Vector        colType;
    private  Vector        numAnswers;  //in given column
    private  Vector        numAnsShown; //in given column
    //both vectors have same index for same column

    private  static final  String  BIGDELIM = "\n";

    public BrowserResponse (ParkaNet pn)
    {
	super (pn);
	allTokens = null;
	numCols = 0;
	colType = new Vector();
	numAnswers = new Vector();
	numAnsShown = new Vector();
	browserResponse = "";
	lastColFinished = true;  //will be true unless client and server
             //need multiple calls to get all data, and stopped in middle
             //of a column
    }

    private void setColType (String[] tokenlist)
    {
	if ((tokenlist != null) && (tokenlist.length > 0))
	    colType.addElement (tokenlist[0]);
	else
	    System.out.println ("Programmer Error:  no list of tokens for "
                  + "this column");
    }

    private void recordSizes (String workstring, String[] tokenlist,
                              int blankloc)
    {
        String  numAns = workstring.substring (0, blankloc);
        numAnswers.addElement (numAns);

        setColType (tokenlist);

        int     numAnsReturned = tokenlist.length -1;
	    /* minus one, because first token is really which col we have*/
        numAnsShown.addElement (String.valueOf(numAnsReturned));
    }

    // Should only be called after super.computeResponse() has been
    // called, else 'response' will be an empty string.
    //
    protected void makeTokens()
    {
	browserResponse = response;  //'response' is in parent class
        StringTokenizer  columnTokens = new StringTokenizer (
                                browserResponse, BIGDELIM);
/* TEST
        System.out.println ("Response is [" + browserResponse + "]");
        System.out.println ("Num columns is " + columnTokens.countTokens());
*/
	allTokens = new String[columnTokens.countTokens()][];

	// get each column of KB data, using parent class's tokenizing
	// routine
        while (columnTokens.hasMoreTokens())
        {
            String workstring = columnTokens.nextToken();
		// The form of this response is: num tokens,
                // column type, then the token list.
		// Blank is the first separator; the rest are \t.
		// The browser will expect the column type, but not
		// the leading number of tokens, so we remove it.
/* TEST
	    System.out.println ("\nCOL_TOKS: <" + workstring + ">");
*/
	    int  blankloc = workstring.indexOf (' ');
	    response = workstring.substring (blankloc +1);

	    super.makeTokens();
	    allTokens[numCols] = tokenList; //result of 'super.makeTokens()'

	    recordSizes (workstring, tokenList, blankloc);
/* TEST
                System.out.println ("Num items in column is " +
                                   allTokens[numCols].length + " long");
*/
            numCols++;
        }
    }

    protected void makeMoreTokens()
    {
	    //from prior call:
	lastColFinished =
                (browserResponse.charAt(browserResponse.length() -1) == '\n');
	browserResponse = response;
        StringTokenizer  columnTokens = new StringTokenizer (
                                browserResponse, BIGDELIM);
/* TEST
//      Warning: the next line prints a lot for large datasets:
//      System.out.println ("MORE: Response is [" + browserResponse + "]");
 
	System.out.println ("\nMORE:  lastColFinished == " + lastColFinished);
	System.out.println ("MORE: Size Of 'browserResponse' is: " +
                        browserResponse.length());
        System.out.println ("MORE: Num big-delim tokens is " +
                                            columnTokens.countTokens());
	System.out.println ("MORE: Num column slots in 'allTokens is: " +
                      allTokens.length);
	for (int i = 0; i < browserResponse.length(); i++)
	{
	  System.out.print ( (int)(browserResponse.charAt(i)) );
	  System.out.print ('-');
	  System.out.print (browserResponse.charAt(i));
	  System.out.print (' ');
	  if (i % 10 == 0)
	      System.out.println ("");
	}
*/
	int  numToks = columnTokens.countTokens();
	     //tokenizer will always have at least one token: the whole string
	if (lastColFinished || (numToks > 1))
	{
	    if (!lastColFinished)
		numToks--;  //want one less column
	    String[][]  tmpAllTokens = allTokens;
	    allTokens = new String[numToks + tmpAllTokens.length][];
	    System.arraycopy (tmpAllTokens, 0, allTokens, 0,
                                            tmpAllTokens.length);
	}
	while (columnTokens.hasMoreTokens())
        {
            String workstring = columnTokens.nextToken();
	         //get rid of leading number in this col:
/* TEST
	    System.out.println ("MORE COL_TOKS: <" + workstring + ">");
*/
	    int  blankloc = -1;
            if (lastColFinished)
	    {
		blankloc = workstring.indexOf (' ');
		response = workstring.substring (blankloc +1);
	    }
	    else
		response = workstring;

	    super.makeTokens();  // uses 'response' to make tokens from
	    if (!lastColFinished)
	    {
/* TEST
                System.out.println ("         Added to existing col");
*/
		numCols--;  //go back to last one
		String tmp[] = allTokens[numCols];
		String newTokenCol[] = new String[tmp.length +
                                                    tokenList.length];
		System.arraycopy (tmp, 0, newTokenCol, 0, tmp.length);
		System.arraycopy (tokenList, 0, newTokenCol, tmp.length,
                                                   tokenList.length);
		allTokens[numCols] = newTokenCol;
		lastColFinished = true;

		// adjust number of items retrieved for this column
		// we subtract one because we are replacing the old number,
		// and don't want to count the column type entry (1st entry)
		// (same reason it's done in 'recordSizes()'.
		//
		numAnsShown.setElementAt (String.valueOf(newTokenCol.length -1),
                                                  numCols);
	    }
	    else
	    {
	        allTokens[numCols] = tokenList; //result of 'super.makeTokens()'
                recordSizes (workstring, tokenList, blankloc);
	    }
/* TEST
            System.out.println ("MORE: Num items in column is " +
                                   allTokens[numCols].length + " long");
*/
	    numCols++;
	}
    }

    public String[] getTokenList()
    {
	System.out.println ("Invalid to call getTokenList() for class "
				+ "BrowserResponse");
	return   null;
    }

    public String[][]  getBrowserTokenList()
    {
	if (tokenList != null)
	    tokenList = null; //give to garbage collector
	return  allTokens;
    }

    public Vector getColType()
    {
	return  colType;
    }

    public Vector getNumAnswers()
    {
	return  numAnswers;
    }

    public Vector getNumAnswersShown()
    {
	return  numAnsShown;
    }
}
