/* Copyright (C) 1999 Jeff Heflin - University of Maryland
 *	
 * This software is provided under the terms of the GNU Lesser General Public
 * License Version 2.1, February 1999. See the header in ShoeKb.java
 * and the file License.html for details.
 */

/* Change History:
   - 05/24/00, JDH: Made this class extend the Sentence class
   - 05/22/00, JDH: Added numAnswersRequested variable and methods
   */
package ShoeKb;

import java.util.*;

/** A query that can be issued to a knowledge base. A query can consist
  of one or more atoms conjuncted together, where an atom is a predicate
  with one or more arguments. An argument may be a literal or a variable,
  where varaibles are indicated by preceding them with a question mark. */
public class Query extends Sentence {

  /** Constant for specifying that all answers should be returned. */
  public static final int ALL_ANSWERS = -1;

  int    numAnswersRequested = ALL_ANSWERS;

  /** Construct a new Query without initializing it. */
  public Query() {

    super();
  }

  /** Construct a new Query which consists of a single predicate with
    any number of arguments. */
  public Query(String predicate, String[] args) {
    
    super(predicate, args);
  }

  /** Constructs a new Query which consists of a single predicate with
    two arguments. */
  public Query(String predicate, String arg1, String arg2) {

    super(predicate, arg1, arg2);
  }

  /** Returns the maximum number of answers that should be returned by
    the query. */
  public int getNumAnswersRequested() {

    return numAnswersRequested;
  }

  /** Sets the maximum number of answers that should be returned by
    the query. Some KBs may choose to ignore this value. */
  public void setNumAnswersRequested(int numAnswersRequested) {

    this.numAnswersRequested = numAnswersRequested;
  }

}

